unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, DSound, afDXSPlay, afDXSUtil;

type
  TfrmMain = class(TForm)
    Label1: TLabel;
    GroupBox1: TGroupBox;
    ListBox1: TListBox;
    btnNew: TButton;
    btnKill: TButton;
    GroupBox2: TGroupBox;
    btnPlay: TButton;
    btnStop: TButton;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    scrlVol: TScrollBar;
    scrlPan: TScrollBar;
    scrlFreq: TScrollBar;
    btnOpen: TButton;
    btnPlayLoop: TButton;
    OpenDialog1: TOpenDialog;
    MemoStatus: TMemo;
    btnStatus: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure btnKillClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure btnPlayLoopClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure scrlVolChange(Sender: TObject);
    procedure scrlPanChange(Sender: TObject);
    procedure scrlFreqChange(Sender: TObject);
    procedure btnStatusClick(Sender: TObject);
  private
    { Private declarations }
    Buff : TafDXSPlayStaticBuffer;
    procedure ReBuildBufferDisplay;
    procedure DisableBufferControls;
    procedure EnableBufferControls;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  // Set up the Playback object
  DXSPlay_Init(Self, Handle, DSSCL_PRIORITY);
  if Assigned(DXSPlay) and DXSPlay.Ready then MemoStatus.Lines.Add('DX Sound Playback available.')
    else MemoStatus.Lines.Add('DX Sound Playback not available.');

  // initialize our objects
  Buff := nil;
  // disable controls unless there's a selected buffer
  DisableBufferControls;

  // initialize scroll bars
  scrlVol.Min := DSBVOLUME_MIN;
  scrlVol.Max := DSBVOLUME_MAX;
  scrlPan.Min := DSBPAN_LEFT;
  scrlPan.Max := DSBPAN_RIGHT;
  scrlFreq.Min := DSBFREQUENCY_MIN;
  scrlFreq.Max := DSBFREQUENCY_MAX;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  DXSPlay_Free; // Free the Playback object
end;

procedure TfrmMain.ReBuildBufferDisplay;
var
  x : longint;
  tempBuff : TafDXSPlayStaticBuffer;
begin
  ListBox1.Items.Clear;
  if Assigned(DXSPlay) then
    for x := 0 to DXSPlay.StaticBuffers.Count -1 do begin
      tempBuff := DXSPlay.StaticBuffers[x];
      ListBox1.Items.Add(tempBuff.Name);
    end;
end;

procedure TfrmMain.DisableBufferControls;
begin
  btnKill.Enabled := False;
  btnPlay.Enabled := False;
  btnStop.Enabled := False;
  btnOpen.Enabled := False;
  btnPlayLoop.Enabled := False;
  scrlVol.Enabled := False;
  scrlPan.Enabled := False;
  scrlFreq.Enabled := False;
end;

procedure TfrmMain.EnableBufferControls;
begin
  if Assigned(Buff) then begin
    btnKill.Enabled := True;
    btnPlay.Enabled := True;
    btnStop.Enabled := True;
    btnOpen.Enabled := True;
    btnPlayLoop.Enabled := True;
    scrlVol.Enabled := True;
    scrlPan.Enabled := True;
    scrlFreq.Enabled := True;

    scrlVol.Position := Buff.Volume;
    scrlPan.Position := Buff.Pan;
    scrlFreq.Position := Buff.Frequency;
  end;
end;

procedure TfrmMain.btnNewClick(Sender: TObject);
var
  BuffName : String;
begin
  if Assigned(DXSPlay) then begin
    BuffName := InputBox('Enter a Buffer Name', '', '');
    DXSPlay.NewStaticBuffer(BuffName, DSBCAPS_CTRLFREQUENCY +
                                      DSBCAPS_CTRLPAN +
                                      DSBCAPS_CTRLVOLUME);
    ReBuildBufferDisplay;
    DisableBufferControls;
  end;
end;

procedure TfrmMain.ListBox1Click(Sender: TObject);
begin
  DisableBufferControls;
  Buff := nil;
  if Assigned(DXSPlay) and (ListBox1.ItemIndex > -1) then
    Buff := DXSPlay.StaticBuffer(ListBox1.Items[ListBox1.ItemIndex]);
  if Assigned(Buff) then EnableBufferControls;
end;

procedure TfrmMain.btnKillClick(Sender: TObject);
begin
  if Assigned(DXSPlay) and (ListBox1.ItemIndex > -1) then begin
    Buff := DXSPlay.StaticBuffer(ListBox1.Items[ListBox1.ItemIndex]);
    if Buff <> nil then Buff.Free;
    ReBuildBufferDisplay;
    DisableBufferControls;
  end;
end;

procedure TfrmMain.btnPlayClick(Sender: TObject);
begin
  if Assigned(Buff) then Buff.Play;
end;

procedure TfrmMain.btnPlayLoopClick(Sender: TObject);
begin
  if Assigned(Buff) then Buff.PlayLoop;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
  if Assigned(Buff) then Buff.Stop;
end;

procedure TfrmMain.btnOpenClick(Sender: TObject);
begin
  if Assigned(Buff) and (OpenDialog1.Execute) then
    Buff.LoadFromWaveFile(OpenDialog1.Filename);
end;

procedure TfrmMain.scrlVolChange(Sender: TObject);
begin
  if Assigned(Buff) then
    Buff.Volume := scrlVol.Position;
end;

procedure TfrmMain.scrlPanChange(Sender: TObject);
begin
  if Assigned(Buff) then Buff.Pan := scrlPan.Position;
end;

procedure TfrmMain.scrlFreqChange(Sender: TObject);
begin
  if Assigned(Buff) then Buff.Frequency := scrlFreq.Position;
end;

procedure TfrmMain.btnStatusClick(Sender: TObject);
begin
  if Assigned(Buff) then begin
    MemoStatus.Lines.Add('Name = ' + Buff.Name);
    MemoStatus.Lines.Add('Status = ' + Buff.StatusDescription);
    MemoStatus.Lines.Add('Volume = ' + IntToStr(Buff.Volume));
    MemoStatus.Lines.Add('Pan = ' + IntToStr(Buff.Pan));
    MemoStatus.Lines.Add('Frequency = ' + IntToStr(Buff.Frequency));
  end;
end;

end.
